// Package Tools
// $Author: Jeffrey J Weimer$
// SVN History: $Revision: 1441 $ on $Date: 2010-09-26 11:45:27 -0700 (Sun, 26 Sep 2010) $

// pragmas

#pragma rtGlobals=1
#pragma IgorVersion=6.20
#pragma version=1.2
#pragma hide=1

#pragma ModuleName=PackageTools

// package parameters

Static StrConstant thePackage="PackageTools"
Static StrConstant thePackageFolder="root:Packages:PackageTools"
Static StrConstant theProcedureFile = "PackageTools.ipf"
Static StrConstant thePackageInfo = "Tools to manage packages"
Static StrConstant thePackageAuthor = "Jeffrey J Weimer"
Static Constant thePackageVersion = 1.2
Static Constant hasHelp = 0
Static Constant removable = 0

Static StrConstant HookPrefix = "PTAfterCompile_*"

// STATIC HOOKS

// after compiled hook

Function AfterCompileHookProto()

	return 0
end

Static Function AfterCompiledHook()

	Initialize()
	
	string theList, theOne, theIMList, theIMOne, theWin
	variable ic=0, nt=0, ni=0
	
	theIMList = IndependentModuleList(";")
	ni = ItemsInList(theIMList)
	if (ni!= 0)
		Execute/Q "setigoroption independentmoduledev=1"
		for (ic=0;ic<ni;ic+=1)
			theIMOne = StringFromList(ic,theIMList)
			sprintf theWin, "WIN:[%s]", theIMOne
			theOne = RemoveEnding(FunctionList(HookPrefix,";",theWin))
			if (strlen(theOne)!=0)
				sprintf theOne, "%s#%s()", theIMOne, theOne
				Execute/Q/Z theOne
				nt += 1
			endif
		endfor
		Execute/Q "setigoroption independentmoduledev=0"
	endif
	
	theList = FunctionList(HookPrefix,";","KIND:7")
	nt = ItemsInList(theList)
	for (ic=0;ic<nt;ic+=1)
		theOne = StringFromList(ic,theList)
		FUNCREF AfterCompileHookProto f = $theOne
		f()
	endfor
	
	return 0

end

// STATIC FUNCTIONS

// initialize Package Tools

Static Function Initialize()

	DFREF pf = $thePackageFolder
	DFREF cdf = GetDataFolderDFR()
	string theCmd
	
	if (DataFolderRefStatus(pf)==0)
		// create package folder
		NewDataFolder/O/S Packages
		NewDataFolder/O/S PackageTools
		string/G logfile = ""
		NewDataFolder/O Keys
		SetDataFolder cdf
		// setup the package
		sprintf theCmd, "PackageSetup(\"%s\",file=\"%s\",info=\"%s\",author=\"%s\",version=%g,hasHelp=%d,removable=%d)", thePackage,theProcedureFile,thePackageInfo,thePackageAuthor,thePackageVersion,hasHelp,removable
	else
		// update the package
		sprintf theCmd, "PackageUpdate(\"%s\",file=\"%s\",info=\"%s\",author=\"%s\",version=%g,hasHelp=%d,removable=%d)", thePackage,theProcedureFile,thePackageInfo,thePackageAuthor,thePackageVersion,hasHelp,removable
	endif
	
	Execute/Q theCmd
	
	return 0
end

// show log

Static Function ShowLog()

	DFREF pf = $thePackageFolder	
	SVAR/SDFR=pf logfile
	
	if (!SVAR_exists(logfile))
		return 0
	endif
	
	if (WinType("PackageToolsLogFile")==5)
		KillWindow PackageToolsLogFile
	endif
	NewNotebook/F=0/K=1/N=PackageToolsLogFile as "Package Tools Log"
	Notebook PackageToolsLogFile, text=logfile
	
	return 0

end

// list packages

Static Function ListPackages()

	string theKeyF = thePackageFolder + ":Keys"

	// any packages set up?
	
	variable np = CountObjects(theKeyF,3)
	
	if (np == 0)
		DoAlert 0, "No packages have been set up"
		return 0
	endif
	
	// generate information for all packages
	
	variable ic
	string theP, theTxt = ""
	
	for (ic=0;ic<np;ic+=1)
		theP = GetIndexedObjName(theKeyF, 3, ic)
		SVAR/SDFR=$theKeyF thePStr = $theP
		theTxt += "--\r" + ReplaceString(";",thePStr,"\r")
	endfor
	
	if (WinType("ListofPackages")==5)
		KillWindow ListofPackages
	endif
	NewNotebook/F=0/K=1/N=ListofPackages as "List of Packages"
	Notebook ListofPackages, text=theTxt
	
	return 0

end

// get a key value for the package

Static Function/S PackageKey(name,key)
	string name, key
	
	DFREF pf = $thePackageFolder	
	DFREF ptdf = pf:Keys
	
	// return blank if package tools not installed
	
	if (DataFolderRefStatus(ptdf) == 0)
		return ""
	endif
	
	// get and return the package key string
	
	SVAR/SDFR=ptdf ptStr=$name
	
	if (SVAR_exists(ptStr))
		return (StringByKey(key,ptStr))
	else
		return ""
	endif
end

// GLOBAL FUNCTIONS

// check if package exists (exists when it has a defined Key string)

// get the DFREF location for a folder in a given package

Function/DF PackageFolderDFR(name,[folder])
	string name
	string folder
	
	string pdf = PackageKey(name,"folder")
	
	// get and return the package folder DFREF
	
	if (strlen(pdf)!=0)
		if (ParamIsDefault(folder))
			sprintf pdf, "root:Packages:%s", pdf
		else
			sprintf pdf, "root:Packages:%s:%s", pdf, folder
		endif
	endif
	
	DFREF pdfR = $pdf
	
	return pdfR
end

Function PackageExists(name)
	string name
	
	DFREF pf = $thePackageFolder
	
	SVAR/Z/SDFR=pf:Keys theKey=$name
	
	variable/G V_exists=SVAR_exists(theKey)
	return V_exists

end

// set up package parameters

Function PackageSetup(name,[folder,version,info,author,file,hasHelp,removable,more,quiet])
	string name, folder, info, author, file, more
	variable version, hasHelp, quiet, removable
	
	// check for package tools install
	
	if (DataFolderRefStatus($thePackageFolder)==0)
		PackageTools#Initialize()
	endif
	
	// local parameters
	
	string thePackage, theFolder
	
	// set the package name and folder
	
	thePackage = CleanUpName(name,0)
	if (ParamIsDefault(folder))
		theFolder = thePackage
	else
		theFolder = CleanUpName(folder,0)
	endif
	
	// initialize the package parameter	

	DFREF cdf = GetDataFolderDFR()
	DFREF ptf = $thePackageFolder
	SetDataFolder ptf
	SVAR logfile
	SetDataFolder :Keys
	
	SVAR/Z ptStr = $thePackage
	if (!SVAR_exists(ptStr))
		string/G $thePackage
		SVAR ptStr = $thePackage
		sprintf ptStr, "package:%s;folder:%s;", thePackage, theFolder
	endif	
	SetDataFolder cdf
	
	// store the remaining package parameter values
	
	if (!ParamIsDefault(file))
		if (strsearch(file,":",0)>0)
			file = ParseFilePath(0,file,":",1,0)
		endif
		ptStr=ReplaceStringByKey("file",ptStr,file)
	endif
	
	if (!ParamIsDefault(info))
		ptStr=ReplaceStringByKey("info",ptStr,info)
	endif
	
	if (!ParamIsDefault(author))
		ptStr=ReplaceStringByKey("author",ptStr,author)
	endif
	
	if (!ParamIsDefault(version))
		ptStr=ReplaceNumberByKey("version",ptStr,version)
	else
		version = 0
	endif
	
	if (!ParamIsDefault(hasHelp))
		ptStr=ReplaceNumberByKey("hasHelp",ptStr,hasHelp)
	endif
	
	if (!ParamIsDefault(removable))
		ptStr=ReplaceNumberByKey("removable",ptStr,removable)
	endif
	
	if (!ParamIsDefault(more))
		ptStr+= more
	endif
	
	// log events
	
	sprintf logfile, "%s%s, %s -- Setup Package: %s %4.2f\r", logfile, date(), time(), thePackage, version

	// done
	
	quiet = ParamIsDefault(quiet) ? 0 : quiet
	
	if (!quiet)
		printf "Setup Package: %s %4.2f\r", thePackage, version
	endif
	
	return 0
	
end

// set up package parameters

Function PackageUpdate(name,[folder,version,info,author,file,hasHelp,removable,more,quiet])
	string name, folder, info, author, file, more
	variable version, hasHelp, quiet, removable
	
	// check for package tools install
	
	if (DataFolderRefStatus($thePackageFolder)==0)
		PackageTools#Initialize()
	endif
	
	// check for package key
	
	DFREF pf = $thePackageFolder
	DFREF ptdf = pf:Keys
	SVAR/Z/SDFR=ptdf ptStr = $name
	if (!SVAR_exists(ptStr))
		return -1
	endif
	
	// update the package parameter values

	variable updated = 0, ic, nItems
	string ckey, kStr, tmpStr
	
	if (!ParamIsDefault(file))
		ckey = PackageKey(name,"file")
		if (cmpstr(file,ckey)!=0)
			if (strsearch(file,":",0)>0)
				file = ParseFilePath(0,file,":",1,0)
			endif
			ptStr=ReplaceStringByKey("file",ptStr,file)
			updated = 1
		endif
	endif
	
	if (!ParamIsDefault(info))
		ckey = PackageKey(name,"info")
		if (strlen(info) != strlen(ckey))
			ptStr=ReplaceStringByKey("info",ptStr,info)
			updated = 1
		endif
	endif
	
	if (!ParamIsDefault(author))
		ckey = PackageKey(name,"author")
		if (strlen(author) != strlen(ckey))
			ptStr=ReplaceStringByKey("author",ptStr,author)
			updated = 1
		endif
	endif
	
	ckey = PackageKey(name,"version")
	if (!ParamIsDefault(version))
		if (cmpstr(num2str(version),ckey)!=0)
			ptStr=ReplaceNumberByKey("version",ptStr,version)
			updated = 1
		endif
	else
		if (strlen(ckey)==0)
			version = 0
		else
			version = str2num(ckey)
		endif
	endif
	
	if (!ParamIsDefault(hasHelp))
		ckey = PackageKey(name,"hasHelp")
		if (cmpstr(num2str(hasHelp),ckey)!=0)
			ptStr=ReplaceNumberByKey("hasHelp",ptStr,hasHelp)
			updated = 1
		endif
	endif
	
	if (!ParamIsDefault(removable))
		ckey = PackageKey(name,"removable")
		if (cmpstr(num2str(removable),ckey)!=0)
			ptStr=ReplaceNumberByKey("removable",ptStr,removable)
			updated = 1
		endif
	endif
	
	if (!ParamIsDefault(more))
		nItems = ItemsinList(more)
		for (ic=0;ic<nItems;ic+=1)
			tmpStr = StringFromList(ic,more)
			kStr = StringFromList(0,tmpStr,":")
			ckey = PackageKey(name,kStr)
			if (strlen(ckey)==0)
				ptStr += tmpStr + ";"
				updated = 1
			else
				tmpStr = StringFromList(1,tmpStr,":")
				if (cmpstr(ckey,tmpStr)!=0)
					ptStr=ReplaceStringByKey(kStr,ptStr,tmpStr)
					updated = 1
				endif
			endif
		endfor
	endif
	
	// log events
	
	SVAR/Z/SDFR=pf logfile
	
	if (updated)
		sprintf logfile, "%s%s, %s -- Updated Package: %s %4.2f\r", logfile, date(), time(), name, version
	endif
	
	// done
	
	quiet = ParamIsDefault(quiet) ? 0 : quiet
	
	if ((!quiet) && updated)
		printf "Updated Package: %s %4.2f\r", name, version
	endif
	
	return 0
	
end

// display help for package

Function PackageHelp(name,[key])
	string name, key

	string hasHelp = PackageKey(name,"hasHelp")
	string otxt
	
	if (ParamIsDefault(key))
		key = ""
		otxt = ""
	else
		otxt = PackageKey(name,key)
	endif
	
	// check for non-registered or non-existent help
	
	if ( strlen(hasHelp)==0 || !cmpstr(hasHelp,"0") )
		hasHelp = PackageKey(name,"info")
		if (strlen(hasHelp)==0)
			sprintf key, "NO HELP OR INFO REGISTERED FOR PACKAGE: %s\r(may not be setup properly)", name
			DoAlert 0, key
		else
			if (strlen(key)==0)
				sprintf key, "NO HELP FILE INCLUDED FOR PACKAGE: %s\rInfo: %s", name, hasHelp
				DoAlert 0, key
			else
				sprintf key, "%s %s: %s\r", name, key, otxt
				DoAlert 0, key	
			endif
		endif
		return 0
	endif

	// show help
	
	DisplayHelpTopic name
	return 0

end

// remove package

Function PackageRemove(name,[quiet])
	string name
	variable quiet
		
	quiet = ParamIsDefault(quiet) ? 0 : quiet

	// package setup?
		
	DFREF pf = $thePackageFolder	
	DFREF ptdf = pf:Keys

	SVAR/Z/SDFR=pf logfile	
	SVAR/Z/SDFR=ptdf pStr = $name
	
	if (!SVAR_exists(pStr))
		if (!quiet)
			DoAlert 0, "Package " + name + " is not set up."
		endif
		return -1
	endif

	string theCmd, theRStr, thePFolder
	variable removable, pexists, removed=0
	
	name = PackageKey(name,"package")
	
	// can the package be removed (in a standard way)?
	
	removable = Str2Num(PackageKey(name,"removable"))
	
	if (numtype(removable)==2)
		if (!quiet)
			DoAlert 0, "Package " + name + " does not support standard removal."
		endif
		return -1
	endif
	if (removable == 0)
		if (!quiet)
			DoAlert 0, "Package " + name + " is marked as UNREMOVABLE!"
		endif
		return -1
	endif
	
	// confirm request if not quiet
	
	if (!quiet)
		DoAlert 1, "Do you really want to remove the package " + name + "?"	
		if (V_flag == 2)
			return 0
		endif
	endif
	
	// kill the package folder
	
	thePFolder = "root:Packages:" + PackageKey(name,"folder")		
	pexists = DataFolderExists(thePFolder) ? 1 : 0
	KillDataFolder/Z $thePFolder
	
	// delete the package include statement
	
	sprintf theCmd "DELETEINCLUDE \"%s\"" name
	Execute/P/Z/Q theCmd

	// close the package procedure file
	
	theCmd = PackageKey(name,"file")
	if (strlen(theCmd)!=0)
		sprintf theCmd "CloseProc/NAME=\"%s\"" theCmd	
		Execute/P/Z/Q theCmd
	endif
	
	sprintf theRStr, "Removed Package: %s" name
	
	if (pexists)
		pexists = DataFolderExists(thePFolder) ? 1 : 0
		if (pexists)
			sprintf theRStr, "%s (Package Folder Still Installed)", theRStr
		endif
	endif
	
	killstrings/Z pStr

	// done
	
	if (!quiet)
		print theRStr
	endif
	
	if (SVAR_exists(logfile))
		sprintf logfile, "%s%s, %s -- %s\r", logfile, date(), time(), theRStr
	endif
	return 0
	
end
